/**
 * Copyright (C) 2011 Ovea <dev@ovea.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ovea.resource.web.util;

import java.io.*;
import java.net.JarURLConnection;
import java.util.Deque;
import java.util.Enumeration;
import java.util.LinkedList;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

/**
 * @author Mathieu Carbou (mathieu.carbou@gmail.com)
 */
public final class Scanner {

    public static void scan(Resource base, Callback callback) {
        if (base.isFile()) {
            Deque<File> files = new LinkedList<File>();
            try {
                String baseDir = base.toFile().getCanonicalFile().getAbsolutePath();
                files.add(new File(baseDir));
                while (!files.isEmpty()) {
                    File file = files.pop();
                    if (file.isDirectory())
                        for (File child : file.listFiles())
                            files.addFirst(child);
                    else {
                        try {
                            String relativePath = file.getCanonicalPath().substring(baseDir.length()).replace("\\", "/");
                            if (relativePath.length() > 0 && !relativePath.startsWith("/"))
                                relativePath = "/" + relativePath;
                            callback.onMatch(relativePath, Resource.from(file));
                        } catch (IOException e) {
                            callback.onError(e);
                        }
                    }
                }
            } catch (IOException e) {
                callback.onError(e);
            }
        } else {
            try {
                String path = base.toURL().getPath();
                path = path.substring(path.indexOf("!") + 2);
                if(!path.endsWith("/")) {
                    path = path + "/";
                }
                JarURLConnection jarCon = (JarURLConnection) base.toURL().openConnection();
                JarFile jarFile = jarCon.getJarFile();
                Enumeration<JarEntry> e = jarFile.entries();
                while (e.hasMoreElements()) {
                    JarEntry jarEntry = e.nextElement();
                    if (!jarEntry.isDirectory() && jarEntry.getName().startsWith(path)) {
                        callback.onMatch(jarEntry.getName().substring(path.length() - 1), Resource.fromData(read(jarFile.getInputStream(jarEntry))));
                    }
                }
            } catch (ResourceException e1) {
                callback.onError(e1);
            } catch (IOException e1) {
                throw new RuntimeException(e1.getMessage(), e1);
            }
        }
    }

    public static interface Callback {
        /**
         * relativePath starts with /
         */
        void onMatch(String relativePath, Resource resource);

        void onError(Throwable t);
    }

    private static byte[] read(InputStream in) throws IOException {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        try {
            in = new BufferedInputStream(in);
            byte[] buffer = new byte[8096];
            int c;
            while ((c = in.read(buffer)) != -1)
                baos.write(buffer, 0, c);
            return baos.toByteArray();
        } finally {
            in.close();
        }
    }

}
